<?php
/*##################################################
 *		    AdminSeriesManageController.class.php
 *                            -------------------
 *   begin                : June 04, 2013
 *   copyright            : (C) 2014 Root
 *   email                : contact@graphismeo.ovh
 *
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

/**
 * @author Patrick DUBEAU <daaxwizeman@gmail.com>
 */
class AdminSeriesManageController extends AdminModuleController
{
	const NUMBER_ITEMS_PER_PAGE = 20;
	
	private $lang;
	private $view;
	
	public function execute(HTTPRequestCustom $request)
	{
		$this->init();
		
		$this->build_view($request);
		
		return new AdminSeriesDisplayResponse($this->view, $this->lang['series_management']);
	}
	
	private function init()
	{
		$this->lang = LangLoader::get('common', 'series');
		$this->view = new FileTemplate('series/AdminSeriesManageController.tpl');
		$this->view->add_lang($this->lang);
	}
	
	private function build_view($request)
	{
		$mode = $request->get_getvalue('sort', 'desc');
		$field = $request->get_getvalue('field', 'date');
		
		$sort_mode = ($mode == 'asc') ? 'asc' : 'desc';
		
		switch ($field)
		{
			case 'cat':
				$sort_field = 'id_category';
				break;
			case 'author':
				$sort_field = 'login';
				break;
			case 'title':
				$sort_field = 'title';
				break;
			case 'published':
				$sort_field = 'published';
				break;
			default:
				$sort_field = 'date_created';
				break;
		}
		
		$page = $request->get_getint('page', 1);
		$pagination = $this->get_pagination($page, $field, $mode);
		
		$result = PersistenceContext::get_querier()->select('SELECT series.*, member.*, notes.average_notes, notes.number_notes, note.note
		FROM '. SeriesSetup::$series_table . ' series
		LEFT JOIN '. DB_TABLE_MEMBER .' member ON member.user_id = series.author_user_id
		LEFT JOIN ' . DB_TABLE_AVERAGE_NOTES . ' notes ON notes.id_in_module = series.id AND notes.module_name = \'series\'
		LEFT JOIN ' . DB_TABLE_NOTE . ' note ON note.id_in_module = series.id AND note.module_name = \'series\' AND note.user_id = ' . AppContext::get_current_user()->get_id() . '
		ORDER BY ' . $sort_field . ' ' . $sort_mode . '
		LIMIT :number_per_page OFFSET :start_limit',
			array(
				'number_per_page' => $pagination->get_number_items_per_page(),
				'start_limit' => $pagination->get_display_from()
		));
		
		while($row = $result->fetch())
		{
			$serie = new Serie();
			$serie->set_properties($row);
			
			$this->view->assign_block_vars('series', $serie->get_tpl_vars());
		}
		$result->dispose();
		
		$this->view->put_all(array(
			'C_PAGINATION' => $pagination->has_several_pages(),
			'C_SERIES' => $result->get_rows_count() > 0,
			'PAGINATION' => $pagination->display(),
			'U_SORT_TITLE_ASC' => SeriesUrlBuilder::manage_series('title', 'asc', $page)->rel(),
			'U_SORT_TITLE_DESC' => SeriesUrlBuilder::manage_series('title', 'desc', $page)->rel(),
			'U_SORT_CATEGORY_ASC' => SeriesUrlBuilder::manage_series('cat', 'asc', $page)->rel(),
			'U_SORT_CATEGORY_DESC' => SeriesUrlBuilder::manage_series('cat', 'desc', $page)->rel(),
			'U_SORT_AUTHOR_ASC' => SeriesUrlBuilder::manage_series('author', 'asc', $page)->rel(),
			'U_SORT_AUTHOR_DESC' => SeriesUrlBuilder::manage_series('author', 'desc', $page)->rel(),
			'U_SORT_DATE_ASC' => SeriesUrlBuilder::manage_series('date', 'asc', $page)->rel(),
			'U_SORT_DATE_DESC' => SeriesUrlBuilder::manage_series('date', 'desc', $page)->rel(),
			'U_SORT_PUBLISHED_ASC' => SeriesUrlBuilder::manage_series('published', 'asc', $page)->rel(),
			'U_SORT_PUBLISHED_DESC' => SeriesUrlBuilder::manage_series('published', 'desc', $page)->rel()
		));
	}
	
	private function get_pagination($page, $sort_field, $sort_mode)
	{
		$series_number = PersistenceContext::get_querier()->count(SeriesSetup::$series_table);
		
		$pagination = new ModulePagination($page, $series_number, self::NUMBER_ITEMS_PER_PAGE);
		$pagination->set_url(SeriesUrlBuilder::manage_series($sort_field, $sort_mode, '%d'));
		
		if ($pagination->current_page_is_empty() && $page > 1)
		{
			$error_controller = PHPBoostErrors::unexisting_page();
			DispatchManager::redirect($error_controller);
		}
		
		return $pagination;
	}
}
?>