<?php
/*##################################################
 *                             SeriesSetup.class.php
 *                            -------------------
 *   begin                : February 27, 2013
 *   copyright            : (C) 2014 Root
 *   email                : contact@graphismeo.ovh
 *
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

/**
 * @author Patrick DUBEAU <daaxwizeman@gmail.com>
 */
class SeriesSetup extends DefaultModuleSetup
{
	public static $series_table;
	public static $series_cats_table;
	
	/**
	 * @var string[string] localized messages
	*/
	private $messages;

	public static function __static()
	{
		self::$series_table = PREFIX . 'series';
		self::$series_cats_table = PREFIX . 'series_cats';
	}

	public function install()
	{
		$this->drop_tables();
		$this->create_tables();
		$this->insert_data();
	}

	public function uninstall()
	{
		$this->drop_tables();
		ConfigManager::delete('series', 'config');
		SeriesService::get_keywords_manager()->delete_module_relations();
	}

	private function drop_tables()
	{
		PersistenceContext::get_dbms_utils()->drop(array(self::$series_table, self::$series_cats_table));
	}

	private function create_tables()
	{
		$this->create_series_table();
		$this->create_series_cats_table();
	}

	private function create_series_table()
	{
		$fields = array(
			'id' => array('type' => 'integer', 'length' => 11, 'autoincrement' => true, 'notnull' => 1),
			'id_category' => array('type' => 'integer', 'length' => 11, 'notnull' => 1, 'default' => 0),
			'picture_url' => array('type' => 'string', 'length' => 255, 'notnull' => 1),
			'title' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'rewrited_title' => array('type' => 'string', 'length' => 250, 'default' => "''"),
			'description' => array('type' => 'text', 'length' => 65000),
			'contents' => array('type' => 'text', 'length' => 65000),
			'number_view' => array('type' => 'integer', 'length' => 11, 'default' => 0),
			'author_user_id' => array('type' => 'integer', 'length' => 11, 'notnull' => 1, 'default' => 0),
			'author_name_displayed' => array('type' => 'boolean', 'notnull' => 1, 'default' => 1),
			'published' => array('type' => 'integer', 'length' => 1, 'notnull' => 1, 'default' => 0),
			'publishing_start_date' => array('type' => 'integer', 'length' => 11, 'notnull' => 1, 'default' => 0),
			'publishing_end_date' => array('type' => 'integer', 'length' => 11, 'notnull' => 1, 'default' => 0),
			'date_created' => array('type' => 'integer', 'length' => 11, 'notnull' => 1, 'default' => 0),
			'date_updated' => array('type' => 'integer', 'length' => 11, 'notnull' => 1, 'default' => 0),
			'notation_enabled' => array('type' => 'boolean', 'notnull' => 1, 'default' => 1),
			'sources' => array('type' => 'text', 'length' => 65000),
			'episode1' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode2' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode3' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode4' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode5' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode6' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode7' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode8' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode9' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode10' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode11' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode12' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode13' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode14' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode15' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode16' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode17' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode18' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode19' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode20' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode21' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode22' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode23' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode24' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode25' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode26' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode27' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode28' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode29' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode30' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode31' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode32' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode33' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode34' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode35' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode36' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode37' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode38' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode39' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episode40' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr1' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr2' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr3' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr4' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr5' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr6' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr7' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr8' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr9' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr10' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr11' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr12' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr13' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr14' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr15' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr16' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr17' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr18' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr19' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr20' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr21' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr22' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr23' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr24' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr25' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr26' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr27' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr28' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr29' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr30' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr31' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr32' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr33' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr34' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr35' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr36' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr37' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr38' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr39' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'episodevostfr40' => array('type' => 'string', 'length' => 250, 'notnull' => 1, 'default' => "''"),
			'acteurs' => array('type' => 'string', 'length' => 300, 'notnull' => 1, 'default' => "''"),
			'duree' => array('type' => 'string', 'length' => 30, 'notnull' => 1, 'default' => "''"),
			'statut' => array('type' => 'string', 'length' => 20, 'notnull' => 1, 'default' => "''"),
			'annee' => array('type' => 'string', 'length' => 4, 'notnull' => 1, 'default' => "''"),

		);
		$options = array(
			'primary' => array('id'),
			'indexes' => array(
				'id_category' => array('type' => 'key', 'fields' => 'id_category'),
				'title' => array('type' => 'fulltext', 'fields' => 'title'),
				'description' => array('type' => 'fulltext', 'fields' => 'description'),
				'contents' => array('type' => 'fulltext', 'fields' => 'contents')
		));
		PersistenceContext::get_dbms_utils()->create_table(self::$series_table, $fields, $options);
	}

	private function create_series_cats_table()
	{
		RichCategory::create_categories_table(self::$series_cats_table);
	}

	private function insert_data()
	{
		$this->messages = LangLoader::get('install', 'series');
		$this->insert_series_cats_data();
		$this->insert_series_data();
	}
	
	private function insert_series_cats_data()
	{
		PersistenceContext::get_querier()->insert(self::$series_cats_table, array(
			'id' => 1,
			'id_parent' => 0,
			'c_order' => 1,
			'auth' => '',
			'rewrited_name' => Url::encode_rewrite($this->messages['default.category.name']),
			'name' => $this->messages['default.category.name'],
			'description' => $this->messages['default.category.description'],
			'image' => '/series/series.png'
		));
	}
        
	private function insert_series_data()
	{
		PersistenceContext::get_querier()->insert(self::$series_table, array(
			'id' => 1,
			'id_category' => 1,
			'picture_url' => '/series/templates/images/default.png',
			'title' => $this->messages['default.serie.title'],
			'rewrited_title' => Url::encode_rewrite($this->messages['default.serie.title']),
			'description' => $this->messages['default.serie.description'],
			'contents' => $this->messages['default.serie.contents'],
			'number_view' => 0,
			'author_user_id' => 1,
			'author_name_displayed' => Serie::AUTHOR_NAME_DISPLAYED,
			'published' => Serie::PUBLISHED_NOW,
			'publishing_start_date' => 0,
			'publishing_end_date' => 0,
			'date_created' => time(),
			'date_updated' => 0,
			'notation_enabled' => Serie::NOTATION_ENABLED,
			'episode1' => 'http://graphismeo.ovh',
			'episode2' => 'http://graphismeo.ovh',
			'episode3' => 'http://graphismeo.ovh',
			'episode4' => 'http://graphismeo.ovh',
			'episode5' => 'http://graphismeo.ovh',
			'episode6' => 'http://graphismeo.ovh',
			'episode7' => 'http://graphismeo.ovh',
			'episode8' => 'http://graphismeo.ovh',
			'episode9' => 'http://graphismeo.ovh',
			'episode10' => 'http://graphismeo.ovh',
			'episode11' => 'http://graphismeo.ovh',
			'episode12' => 'http://graphismeo.ovh',
			'episode13' => 'http://graphismeo.ovh',
			'episodevostfr1' => 'http://graphismeo.ovh',
			'episodevostfr2' => 'http://graphismeo.ovh',
			'episodevostfr3' => 'http://graphismeo.ovh',
			'episodevostfr4' => 'http://graphismeo.ovh',
			'episodevostfr5' => 'http://graphismeo.ovh',
			'episodevostfr6' => 'http://graphismeo.ovh',
			'episodevostfr7' => 'http://graphismeo.ovh',
			'episodevostfr8' => 'http://graphismeo.ovh',
			'episodevostfr9' => 'http://graphismeo.ovh',
			'episodevostfr10' => 'http://graphismeo.ovh',
			'episodevostfr11' => 'http://graphismeo.ovh',
			'episodevostfr12' => 'http://graphismeo.ovh',
			'episodevostfr13' => 'http://graphismeo.ovh',
			'acteurs' => 'Root',
			'duree' => '42 minutes',
			'annee' => '2014',
			'statut' => 'En production',
			'sources' => serialize(array())
		));
	}
}
?>