<?php
/*##################################################
 *		       SeriesDisplaySeriesController.class.php
 *                            -------------------
 *   begin                : April 03, 2013
 *   copyright            : (C) 2014 Root
 *   email                : contact@graphismeo.ovh
 *
 *
 ###################################################
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 ###################################################*/

class SeriesDisplaySeriesController extends ModuleController
{
	private $lang;
	private $tpl;
	private $serie;
	private $category;
	
	public function execute(HTTPRequestCustom $request)
	{
		$this->check_authorizations();
		
		$this->init();
		
		$this->check_pending_serie();
		
		$this->build_view($request);
		
		return $this->generate_response();
	}
	
	private function init()
	{
		$this->lang = LangLoader::get('common', 'series');
		$this->tpl = new FileTemplate('series/SeriesDisplaySeriesController.tpl');
		$this->tpl->add_lang($this->lang);
	}
	
	private function get_serie()
	{
		if ($this->serie === null)
		{
			$id = AppContext::get_request()->get_getint('id', 0);
			if (!empty($id))
			{
				try 
				{
					$this->serie = SeriesService::get_serie('WHERE series.id=:id', array('id' => $id));
				} 
				catch (RowNotFoundException $e) 
				{
					$error_controller = PHPBoostErrors::unexisting_page();
   					DispatchManager::redirect($error_controller);
				}
			}
			else
				$this->serie = new Serie();
		}
		return $this->serie;
	}
	
	private function check_pending_serie()
	{
		if (!$this->serie->is_published())
		{
			$this->tpl->put('MSG', MessageHelper::display($this->lang['series.not_published'], MessageHelper::WARNING));
		}
		else
		{
			$this->serie->set_number_view($this->serie->get_number_view() + 1);
			SeriesService::update_number_view($this->serie);
		}
	}
	private function build_view(HTTPRequestCustom $request)
	{
		$current_page = $request->get_getint('page', 1);
		$comments_enabled = SeriesConfig::load()->are_comments_enabled();
		
		$this->category = SeriesService::get_categories_manager()->get_categories_cache()->get_category($this->serie->get_id_category());
		
		$serie_contents = $this->serie->get_contents();
		
		//If serie doesn't begin with a page, we insert one
		if (substr(trim($serie_contents), 0, 6) != '[page]')
		{
			$serie_contents = '[page]&nbsp;[/page]' . $serie_contents;
		}
		
		//Removing [page] bbcode
		$serie_contents_clean = preg_split('`\[page\].+\[/page\](.*)`Us', $serie_contents, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
		
		//Retrieving pages 
		preg_match_all('`\[page\]([^[]+)\[/page\]`U', $serie_contents, $array_page);

		$nbr_pages = count($array_page[1]);
		
		if ($nbr_pages > 1)
			$this->build_form($array_page, $current_page);
		
				$this->build_keywords_view();
		
		$page_name = (isset($array_page[1][$current_page-1]) && $array_page[1][$current_page-1] != '&nbsp;') ? $array_page[1][($current_page-1)] : '';
		$this->tpl->put_all(array_merge($this->serie->get_tpl_vars(), array(
			'C_COMMENTS_ENABLED' => $comments_enabled,
			'KERNEL_NOTATION' => NotationService::display_active_image($this->serie->get_notation()),
			'CONTENTS' => isset($serie_contents_clean[$current_page-1]) ? FormatingHelper::second_parse($serie_contents_clean[$current_page-1]) : '',
			'PAGE_NAME' => $page_name,
			'U_EDIT_SERIE' => $page_name !== '' ? SeriesUrlBuilder::edit_serie($this->serie->get_id(), $current_page)->rel() : SeriesUrlBuilder::edit_serie($this->serie->get_id())->rel(),
		)));
		
		$this->build_pages_pagination($current_page, $nbr_pages, $array_page);
		
		//Affichage commentaires
		if ($comments_enabled)
		{
			$comments_topic = new SeriesCommentsTopic($this->serie);
			$comments_topic->set_id_in_module($this->serie->get_id());
			$comments_topic->set_url(SeriesUrlBuilder::display_comments_serie($this->category->get_id(), $this->category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title()));

			$this->tpl->put('COMMENTS', $comments_topic->display());
		}
	}
	
	private function build_form($array_page, $current_page)
	{
		$form = new HTMLForm(__CLASS__, '', false);
		$form->set_css_class('options');
		
		$fieldset = new FormFieldsetHorizontal('pages', array('description' => $this->lang['series.summary']));
		
		$form->add_fieldset($fieldset);
		
		$serie_pages = $this->list_serie_pages($array_page);
		
		$fieldset->add_field(new FormFieldSimpleSelectChoice('serie_pages', '', $current_page, $serie_pages,
			array('class' => 'summary', 'events' => array('change' => 'document.location = "' . SeriesUrlBuilder::display_serie($this->category->get_id(), $this->category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title())->rel() . '" + HTMLForms.getField("serie_pages").getValue();'))
		));
		
		$this->tpl->put('FORM', $form->display());
	}
	
	private function build_pages_pagination($current_page, $nbr_pages, $array_page)
	{
		if ($nbr_pages > 1)
		{	
			$pagination = $this->get_pagination($nbr_pages, $current_page);
			
			if ($current_page > 1 && $current_page <= $nbr_pages)
			{
				$previous_page = SeriesUrlBuilder::display_serie($this->category->get_id(), $this->category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title())->rel() . ($current_page - 1);
				
				$this->tpl->put_all(array(
					'U_PREVIOUS_PAGE' => $previous_page,
					'L_PREVIOUS_TITLE' => $array_page[1][$current_page-2]
				));
			}

			if ($current_page > 0 && $current_page < $nbr_pages)
			{
				$next_page = SeriesUrlBuilder::display_serie($this->category->get_id(), $this->category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title())->rel() . ($current_page + 1);
				
				$this->tpl->put_all(array(
					'U_NEXT_PAGE' => $next_page,
					'L_NEXT_TITLE' => $array_page[1][$current_page]
				));
			}
			
			$this->tpl->put_all(array(
				'C_PAGINATION' => true,
				'C_PREVIOUS_PAGE' => ($current_page != 1) ? true : false,
				'C_NEXT_PAGE' => ($current_page != $nbr_pages) ? true : false,
				'PAGINATION_SERIES' => $pagination->display()
			));
		}
	}
	
	private function list_serie_pages($array_page)
	{
		$options = array();
		
		$i = 1;
		foreach ($array_page[1] as $page_name)
		{
			$options[] = new FormFieldSelectChoiceOption($page_name, $i++);
		}
		
		return $options;
	}
	
		private function build_keywords_view()
	{
		$keywords = $this->serie->get_keywords();
		$nbr_keywords = count($keywords);
		$this->tpl->put('C_KEYWORDS', $nbr_keywords > 0);
		
		$i = 1;
		foreach ($keywords as $keyword)
		{
			$this->tpl->assign_block_vars('keywords', array(
				'C_SEPARATOR' => $i < $nbr_keywords,
				'NAME' => $keyword->get_name(),
				'URL' => SeriesUrlBuilder::display_tag($keyword->get_rewrited_name())->rel(),
			));
			$i++;
		}
	}
	private function check_authorizations()
	{
		$serie = $this->get_serie();
		
		$auth_write = SeriesAuthorizationsService::check_authorizations($serie->get_id_category())->write();
		$auth_moderation = SeriesAuthorizationsService::check_authorizations($serie->get_id_category())->moderation();
		
		$not_authorized = !$auth_moderation && (!$auth_write && $serie->get_author_user()->get_id() != AppContext::get_current_user()->get_id());
		
		switch ($serie->get_publishing_state()) 
		{
			case Serie::PUBLISHED_NOW:
				if (!SeriesAuthorizationsService::check_authorizations()->read() && $not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
		   			DispatchManager::redirect($error_controller);
				}
			break;
			case Serie::NOT_PUBLISHED:
				if ($not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
		   			DispatchManager::redirect($error_controller);
				}
			break;
			case Serie::PUBLISHED_DATE:
				if (!$serie->is_published() && $not_authorized)
				{
					$error_controller = PHPBoostErrors::user_not_authorized();
		   			DispatchManager::redirect($error_controller);
				}
			break;
			default:
				$error_controller = PHPBoostErrors::unexisting_page();
				DispatchManager::redirect($error_controller);
			break;
		}
	}
	
	private function get_pagination($nbr_pages, $current_page)
	{
		$pagination = new ModulePagination($current_page, $nbr_pages, 1, Pagination::LIGHT_PAGINATION);
		$pagination->set_url(SeriesUrlBuilder::display_serie($this->category->get_id(), $this->category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title(), '%d'));
		
		if ($pagination->current_page_is_empty() && $current_page > 1)
		{
			$error_controller = PHPBoostErrors::unexisting_page();
			DispatchManager::redirect($error_controller);
		}
		
		return $pagination;
	}
	
	private function generate_response()
	{
		$response = new SiteDisplayResponse($this->tpl);

		$graphical_environment = $response->get_graphical_environment();
		$graphical_environment->set_page_title($this->serie->get_title());
		$graphical_environment->get_seo_meta_data()->set_description($this->serie->get_description());
		$graphical_environment->get_seo_meta_data()->set_keywords($this->serie->get_keywords_name());
		$graphical_environment->get_seo_meta_data()->set_canonical_url(SeriesUrlBuilder::display_serie($this->category->get_id(), $this->category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title(), AppContext::get_request()->get_getint('page', 1)));
	
		$breadcrumb = $graphical_environment->get_breadcrumb();
		$breadcrumb->add($this->lang['series'], SeriesUrlBuilder::home());
		
		$categories = array_reverse(SeriesService::get_categories_manager()->get_parents($this->serie->get_id_category(), true));
		foreach ($categories as $id => $category)
		{
			if ($category->get_id() != Category::ROOT_CATEGORY)
				$breadcrumb->add($category->get_name(), SeriesUrlBuilder::display_category($category->get_id(), $category->get_rewrited_name()));
		}
		$breadcrumb->add($this->serie->get_title(), SeriesUrlBuilder::display_serie($category->get_id(), $category->get_rewrited_name(), $this->serie->get_id(), $this->serie->get_rewrited_title()));
		
		return $response;
	}
}
?>